// (C) 2009 THQ Inc. Not for commercial use or redistribution without permission. All rights reserved.

#include "AStarTemplate.h"
#include "GridTouchedMap.h"
#include "PathGridEnvironment.h"
#include "TerrainMap.h"
#include "GridTouchedMap.h"
#include "RegionPool.h"
#include "Point2i.h"

int main(int argc, char* argv[])
{
	static int k_mapSize = 32;
	TerrainMap terrainMap(k_mapSize, k_mapSize);

	// Make a wall down the middle of the map
	for(int y = 0; y < k_mapSize - 4; ++y)
	{
		terrainMap.SetPassable(Point2i(k_mapSize / 2, y), 0);
	}

	GridTouchedMap touchedMap;
	touchedMap.SetMapSize(k_mapSize, k_mapSize);

	RegionPool nodePool;

	PathGridEnvironment environment( &terrainMap );

	AStar<Point2i, PathGridEnvironment, GridTouchedMap, float> astar( touchedMap, nodePool );

	printf("Starting search...\n");

	Point2i start(0,0);
	Point2i end(31,0);

	AStarSearchResult result = astar.DoFindPath(start, end, environment, 256);

	switch(result)
	{
	case AStarNoPath:
		printf("Found no path!\n");
		break;

	case AStarGotPathToGoal:
		printf("Found path to goal!\n");
		break;
	
	case AStarGotPathImperfect:
		printf("Found a partial path!\n");
		break;
	}

	int nodesClosed = astar.GetNodesClosed();

	printf("Nodes closed: %d\n", nodesClosed);

	printf("Path:\n");
	for( Point2i* i = astar.GetSolutionStart(); i != 0; i = astar.GetSolutionNext() )
	{
		printf("  Path point: (%d , %d)\n", i->x, i->y);
	}
	printf("End of Path\n");

	return 0;
}

