// (C) 2009 THQ Inc. Not for commercial use or redistribution without permission. All rights reserved.

#include "GridTouchedMap.h"
#include <assert.h>

GridTouchedMap::GridTouchedMap( )
	: m_nextCellToClear( 0 ),
	  m_cellsToClearPerPass( 1 ),
	  m_mapWidth( 0 ),
	  m_sequenceNum( 0 )
{
}

void GridTouchedMap::SetMapSize( unsigned int width, unsigned int height )
{
	size_t newSize = width * height;

	assert(width > 0);
	assert(height > 0);

	if(m_touchedList.size() != newSize)
	{
		m_mapWidth = width;
		m_touchedList.clear();
		m_touchedList.resize(newSize);
		m_nextCellToClear = 0;
		m_cellsToClearPerPass = std::min((newSize / (USHRT_MAX - 1)) + 2, newSize);
		m_sequenceNum = 0;
	}
}

void GridTouchedMap::IncrementalClear( )
{
	iterator i = m_touchedList.begin() + m_nextCellToClear;
	for(size_t cellsCleared = 0; cellsCleared < m_cellsToClearPerPass; ++cellsCleared )
	{
		// Change the cell's sequenceNum to the current.
		i->m_astarNode = 0;
		i->m_sequenceNum = m_sequenceNum;
		i->m_bOpen = false;

		if(++i == m_touchedList.end())
		{
			i = m_touchedList.begin();
		}
	}

	m_nextCellToClear = i - m_touchedList.begin();

	++m_sequenceNum;
}

void GridTouchedMap::StartNewSearch( )
{
	IncrementalClear();

#ifdef RELIC_DEBUG
	for(const_iterator i = m_touchedList.begin(); i != m_touchedList.end(); ++i)
	{
		dbAssert(i->m_sequenceNum != m_sequenceNum);
	}
#endif
}
