// (C) 2009 THQ Inc. Not for commercial use or redistribution without permission. All rights reserved.

#pragma once

#ifndef GRIDTOUCHEDMAP_H
#define GRIDTOUCHEDMAP_H

#include "Point2i.h"
#include <vector>

class  GridTouchedMap
{
public:

	typedef Point2i UserType;
	
	struct TouchedNode
	{
		TouchedNode() : m_astarNode( 0 ), m_bOpen( 0 ), m_sequenceNum(0) { }
		TouchedNode( void* node, unsigned short sequenceNum, bool bOpen )
			: m_astarNode( node ), m_sequenceNum( sequenceNum ), m_bOpen( bOpen ) { }

		void*			m_astarNode;	// If this is null, the node is untouched.
		unsigned short	m_sequenceNum;	// This node is untouched if this is not equal to the current sequence
		bool			m_bOpen;		// True means open, false means closed.
	};


private:

	typedef std::vector<TouchedNode> TouchedList;

public:

	typedef TouchedList::iterator iterator;
	typedef TouchedList::const_iterator const_iterator;

	GridTouchedMap( );

	// This must be called before the first search
	void SetMapSize( unsigned int width, unsigned int height );

	// This must be called at the start of a new A* to empty out old data
	void StartNewSearch();

	void Push( const UserType& userType, void* astarNode, bool bOpen );
	void Update( iterator it, bool bOpen );
	iterator Find( const UserType& userType );
	const_iterator Begin() const;
	const_iterator End() const;
	void* GetNode( const_iterator it ) const;

	// Used to walk map in debug displays
	UserType GetUserType( const_iterator it ) const;
	bool IsValid( const_iterator it ) const;

private:

	unsigned int GetKey( const UserType &pos ) const;
	void IncrementalClear( );

#pragma warning( push )
#pragma warning( disable : 4251 )			// dll-interface

	TouchedList					m_touchedList;
	size_t						m_nextCellToClear;
	size_t						m_cellsToClearPerPass;
	unsigned int				m_mapWidth;
	unsigned short				m_sequenceNum;

#pragma warning( pop )
};

#include "GridTouchedMap.inl"

#endif