// (C) 2009 THQ Inc. Not for commercial use or redistribution without permission. All rights reserved.

#pragma once

#ifndef GRIDTOUCHEDMAP_INL
#define GRIDTOUCHEDMAP_INL

#include <assert.h>

// Helpers come first to assist the compiler with inlining
__forceinline
unsigned int GridTouchedMap::GetKey( const UserType &pos ) const
{
	assert(pos.x < int(m_mapWidth));

	return (pos.y * m_mapWidth) + pos.x;
}

__forceinline
void GridTouchedMap::Push( const UserType& userType, void* astarNode, bool bOpen )
{
	unsigned int key = GetKey( userType );

#ifdef RELIC_DEBUG
	dbAssert(astarNode);
	dbAssert(key < m_touchedList.size());
#endif

	TouchedNode& touchedNode = m_touchedList[key];

	touchedNode.m_astarNode = astarNode;
	touchedNode.m_sequenceNum = m_sequenceNum;
	touchedNode.m_bOpen = bOpen;
}

__forceinline
void GridTouchedMap::Update( iterator it, bool bOpen )
{
#ifdef RELIC_DEBUG
	dbAssert(it->m_astarNode != 0);
	dbAssert(it->m_sequenceNum == m_sequenceNum);
#endif
	it->m_bOpen = bOpen;
}

__forceinline
GridTouchedMap::iterator GridTouchedMap::Find( const UserType& userType )
{
	unsigned int key = GetKey( userType );

#ifdef RELIC_DEBUG
	dbAssert(key < m_touchedList.size());
#endif

	iterator it = m_touchedList.begin() + key;

#ifdef RELIC_DEBUG
	dbAssert ((it->m_sequenceNum != m_sequenceNum) || (it->m_astarNode != 0));
#endif

	return (it->m_sequenceNum == m_sequenceNum) ? it : m_touchedList.end();
}

__forceinline
GridTouchedMap::const_iterator GridTouchedMap::Begin() const
{
	return m_touchedList.begin();
}

__forceinline
GridTouchedMap::const_iterator GridTouchedMap::End() const
{
	return m_touchedList.end();
}

__forceinline
void* GridTouchedMap::GetNode( const_iterator it ) const
{
#ifdef RELIC_DEBUG
	dbAssert(it->m_astarNode);
#endif

	return it->m_astarNode;
}

__forceinline
GridTouchedMap::UserType GridTouchedMap::GetUserType( const_iterator it ) const
{
	size_t key = it - m_touchedList.begin();

	size_t y = key / m_mapWidth;
	size_t x = key - y * m_mapWidth;

	return UserType(x, y);
}

__forceinline
bool GridTouchedMap::IsValid( const_iterator it ) const
{
	return it->m_sequenceNum == m_sequenceNum;
}

#endif
