// (C) 2009 THQ Inc. Not for commercial use or redistribution without permission. All rights reserved.

#include "PathGridEnvironment.h"
#include "PathUtil.h"
#include "TerrainMap.h"

#pragma warning( disable : 4127 ) // warning C4127: conditional expression is constant

//
//
PathGridEnvironment::PathGridEnvironment(	const TerrainMap* terrainMap ) : 
	m_terrainMap( terrainMap ),
	m_width(terrainMap->GetWidth()),
	m_height(terrainMap->GetHeight())
{
}



//
// Heuristic is base on a simple diagonal value plus hor/vert length
//

PathGridCostType PathGridEnvironment::Heuristic( const Point2i &start, const Point2i &goal, PathGridCostType gval ) const
{
	int xdir = abs(goal.x - start.x);
	int zdir = abs(goal.y - start.y);

	PathGridCostType cost;

	if(xdir < zdir)
	{
		cost = (zdir*PPUtil::CARDINAL_COST) + xdir * PPUtil::DIAGONAL_COST_MINUS_CARDINAL;
	}
	else
	{
		cost = (xdir*PPUtil::CARDINAL_COST) + zdir * PPUtil::DIAGONAL_COST_MINUS_CARDINAL;
	}

	// the -gval*GVAL_TIEBREAKER_SCALE is a tiebreaker (when there is a tie between two paths, it prefers to expand the longest one
	return cost * PPUtil::H_TIEBREAKER_SCALE;//  - (gval*GVAL_TIEBREAKER_SCALE);
}



//
// Returns all possible neighbours for the given point.
//
void PathGridEnvironment::GetNeighbours( const Point2i &parent_point, const Point2i &point, std::vector< Point2i > &neighbours, std::vector< PathGridCostType > &gcost ) const
{
	PPUtil::AddNeighbours< PathGridCostType, true, true, true, false, true, false, true >( parent_point, point, neighbours, gcost, m_terrainMap );
}


//
// Test the exit condition. In this case it's looking if the current_point is the goal
//
bool PathGridEnvironment::TestExitCondition( const Point2i &current_point, const Point2i &goal ) const
{
	return( current_point == goal );
}
