// (C) 2009 THQ Inc. Not for commercial use or redistribution without permission. All rights reserved.

#pragma once

#ifndef INCLUDED_RELIC_POINT2I_H
#define INCLUDED_RELIC_POINT2I_H

class Point2i
{
public:

	// Elements
#pragma warning(push)
#pragma warning(disable: 4201)
	union
	{
		int e[2];
		struct
		{
			int x, y;
		};
	};
#pragma warning(pop)

    // Construction / assignment
	Point2i();
	Point2i( int x, int y );

	Point2i& Set( int x, int y );

	// Access
	int  operator[]( size_t i ) const;
	int& operator[]( size_t i );

	// Assignment operators
	Point2i& operator+=( const Point2i& rhs );
	Point2i& operator-=( const Point2i& rhs );

	Point2i& operator*=( int rhs );

	// Binary operators
	friend const Point2i operator+( const Point2i& lhs, const Point2i& rhs );
	friend const Point2i operator-( const Point2i& lhs, const Point2i& rhs );

	friend const Point2i operator*( const Point2i& lhs, int rhs );
	friend const Point2i operator*( int lhs, const Point2i& rhs );

	friend bool operator==( const Point2i& lhs, const Point2i& rhs );
	friend bool operator!=( const Point2i& lhs, const Point2i& rhs );
};



///////////////////////////////////////////////////////////////////////////////
//
// Constructors
//
///////////////////////////////////////////////////////////////////////////////

__forceinline Point2i::Point2i()
{
}

__forceinline Point2i::Point2i( int x_, int y_ ) :
	x( x_ ), y( y_ )
{
}

__forceinline Point2i& Point2i::Set( int x_, int y_ )
{
	x = x_;
	y = y_;

	return *this;
}


///////////////////////////////////////////////////////////////////////////////
//
// Access
//
///////////////////////////////////////////////////////////////////////////////

__forceinline int Point2i::operator[]( size_t i ) const
{
	return e[ i ];
}

__forceinline int& Point2i::operator[]( size_t i )
{
	return e[ i ];
}


///////////////////////////////////////////////////////////////////////////////
//
// Assignment Operators
//
///////////////////////////////////////////////////////////////////////////////

__forceinline Point2i& Point2i::operator+=( const Point2i& rhs )
{
    x += rhs.x; y += rhs.y;
	return *this;
}

__forceinline Point2i& Point2i::operator-=( const Point2i& rhs )
{
    x -= rhs.x; y -= rhs.y;
	return *this;
}

__forceinline Point2i& Point2i::operator*=( int rhs )
{
    x *= rhs; y *= rhs;
	return *this;
}

///////////////////////////////////////////////////////////////////////////////
//
// Binary operators
//
///////////////////////////////////////////////////////////////////////////////

__forceinline const Point2i operator+( const Point2i& lhs, const Point2i& rhs )
{
	Point2i r( lhs );
	return r += rhs;
}

__forceinline const Point2i operator-( const Point2i& lhs, const Point2i& rhs )
{
	Point2i r( lhs );
	return r -= rhs;
}

__forceinline const Point2i operator*( const Point2i& lhs, int rhs )
{
	Point2i r( lhs );
	return r *= rhs;
}

__forceinline const Point2i operator*( int lhs, const Point2i& rhs )
{
	return operator*( rhs, lhs );
}



__forceinline bool operator==( const Point2i& lhs, const Point2i& rhs )
{
	return (lhs.x == rhs.x && lhs.y == rhs.y);
}

__forceinline bool operator!=( const Point2i& lhs, const Point2i& rhs )
{
	return (lhs.x != rhs.x || lhs.y != rhs.y);
}

#endif
