// (C) 2009 THQ Inc. Not for commercial use or redistribution without permission. All rights reserved.

#pragma once

#ifndef INCLUDED_RELIC_REGIONPOOL_H
#define INCLUDED_RELIC_REGIONPOOL_H

#include <vector>
#include <assert.h>

// *** MAY ONLY BE USED TO ALLOCATE PODS!  DESTRUCTORS NOT CALLED BY CLEAR! ***
// *** ALLOC SIZE MUST BE < k_PageAllocSize ***

// This pool is used to quickly allocate pod's that are all freed at one time, such as A* nodes

class RegionPool
{
public:
	RegionPool() : m_frontPage(new RegionPage)
	{
		m_currentPage = m_frontPage;
		m_currentAllocPoint = m_frontPage->m_pageData;
		m_currentAllocEnd = m_currentAllocPoint + k_PageAllocSize;
	}

	~RegionPool()
	{
		RegionPage* i = m_frontPage;
		while(i != 0)
		{
			RegionPage* t = i;
			i = t->m_pNextPage;
			delete t;
		}

		m_frontPage = 0;
		m_currentPage = 0;
	}

	void* Allocate(size_t size)
	{
		assert(size <= k_PageAllocSize);
		assert(size > 0);

		char* newAllocPoint = m_currentAllocPoint + size;
		if( newAllocPoint > m_currentAllocEnd )
		{
			NextPage();
			newAllocPoint = m_currentAllocPoint + size;
		}

		char* allocation = m_currentAllocPoint;
		m_currentAllocPoint = newAllocPoint;

		return allocation;
	}
	
	void Clear()
	{
		m_currentPage = m_frontPage;
	}

private:
	static const size_t k_PageSize = 65536;
	static const size_t k_PageAllocSize = k_PageSize - sizeof(void*);

	struct RegionPage
	{
		RegionPage() : m_pNextPage(0) {}
		RegionPage* m_pNextPage;
		char m_pageData[k_PageAllocSize];
	};

	void NextPage()
	{
		if(m_currentPage->m_pNextPage == 0)
		{
			Extend();
		}
		else
		{
			m_currentPage = m_currentPage->m_pNextPage;
			m_currentAllocPoint = m_currentPage->m_pageData;
			m_currentAllocEnd = m_currentAllocPoint + k_PageAllocSize;
		}
	}

	void Extend()
	{
		assert(m_currentPage->m_pNextPage == 0);
		m_currentPage->m_pNextPage = new RegionPage;
		m_currentPage = m_currentPage->m_pNextPage;
		m_currentAllocPoint = m_currentPage->m_pageData;
		m_currentAllocEnd = m_currentAllocPoint + k_PageAllocSize;
	}

	RegionPage* m_frontPage;
	RegionPage* m_currentPage;
	char*		m_currentAllocPoint;
	char*		m_currentAllocEnd;
};

#endif