// (C) 2009 THQ Inc. Not for commercial use or redistribution without permission. All rights reserved.

#pragma once

#ifndef INCLUDED_RELIC_TERRAINMAP_H
#define INCLUDED_RELIC_TERRAINMAP_H

#include "Point2i.h"
#include <assert.h>
#include <memory.h>

// Rough example map implementation
class TerrainMap
{
public:
	TerrainMap(int width, int height) : m_width(width), m_height(height), m_pData(new char[width*height])
	{
		memset(m_pData, 1, width * height);
	}

	~TerrainMap()
	{
		delete[] m_pData;
		m_pData = 0;
	}

	__forceinline bool IsPassable(const Point2i& p) const 
	{
		if(p.x < 0 || p.y < 0 || p.x >= m_width || p.y >= m_height)
		{
			return false;
		}
		else
		{
			return m_pData[p.y * m_width + p.x] != 0;
		}
	}

	void SetPassable(Point2i& p, char isPassable)
	{
		assert(p.x >= 0 && p.y >= 0 && p.x < m_width && p.y < m_height);

		m_pData[p.y * m_width + p.x] = isPassable;
	}

	__forceinline int GetWidth() const
	{
		return m_width;
	}

	__forceinline int GetHeight() const
	{
		return m_height;
	}

private:
	char* m_pData;
	int m_width;
	int m_height;
};

#endif